import java.util.*;
import java.io.*;

public class HexGame {
	public static void main(String[] args) throws FileNotFoundException { 
		// Start with filename 
		String filename = "C-small-practice.in";

		// Load each test case one at a time
		Scanner s = new Scanner(new File(filename));
		int T = Integer.parseInt(s.nextLine());

		for(int c = 0; c < T; c++) { 

			int n = Integer.parseInt(s.nextLine());

			// Pass the test case to HexBoard constructor
			char[][] board = new char[n][n];
			for(int i = 0; i < n; i++) {
				String row = s.nextLine();
				board[i] = row.toCharArray();
			}

			HexBoard h = new HexBoard(n, board);
			h.link();
			h.printResult(c+1);
		}
	}
}

class HexBoard {
	int M;
	Node[] blueRoot;
	Node[] blueLeaf;
	Node[] redRoot;
	Node[] redLeaf;
	Node[][] hexboard;
	int R = 1, B = -1;

	public HexBoard(int n, char[][] board) {
		this.M = n;
		this.blueRoot = new Node[n];
		this.blueLeaf = new Node[n];
		this.redRoot = new Node[n];
		this.redLeaf = new Node[n];
		this.hexboard = new Node[n][n];

		// Create Node objects
		for(int i=0; i<n; i++) { 
			for(int j=0; j<n; j++) { 
				Node node = new Node(i,j,this);
				if(board[i][j]=='B'){
					node.state = B;
				} else if(board[i][j]=='R') { 
					node.state = R;
				} else {
					node.state = 0;
				}
				hexboard[i][j] = node;
			}
		}
	}

	public void link() {
		// Link Node objects
		for(int i=0; i<M; i++) { 
			for(int j=0; j<M; j++) { 
				Node linkme = hexboard[i][j];
				linkme.link();
			}
		}
	}

	public Node getNode(int row, int col) { 
		if(row >= 0 && row < M && col >= 0 && col < M) { 
			return hexboard[row][col];
		} else {
			return null;
		}
	}

	public boolean isValid() {
		int blues = 0, reds = 0;
		for(int i = 0; i<M; i++) { 
			for(int j=0; j<M; j++) { 
				if( hexboard[i][j].state < 0 ) { 
					blues++;
				} else if( hexboard[i][j].state > 0 ) {
					reds++;
				}
			}
		}
		if(Math.abs(blues-reds)>1) {
			return false;
		} else {
			return true;
		}
	}

	public boolean hasParallelSolutions(int state) {
		// Game should end when one side wins,
		// so look for solutions where one side won
		// and the game kept going.

		// Iterate over flow direction,
		// count up number of adjacent stones in orthog direction.
		int[][] intboard = new int[M][M];
		for( int i=0; i<M; i++ ) {
			for( int j=0; j<M; j++ ) { 
				if( getNode(i,j).state==state ) {
					intboard[i][j] = 1;
				}
			}
		}
		int[] adjMat = new int[M];
		for( int i=0; i<M; i++ ) {
			boolean sw = false;
			int adj = -1;
			for( int j=0; j<M; j++ ) {
				if(state==R) { 
					if(intboard[j][i]==1) { 
						sw = true;
						adj++;
					} else {
						sw = false;
						adj = -1;
					}
				} else {
					if(intboard[i][j]==1) { 
						sw = true;
						adj++;
					} else {
						sw = false;
						adj = -1;
					}
				}
			}
			if(adj>0) {
				adjMat[i] = 1;
			}
		}

		boolean result = false;
		int sum = 0;
		for( int a : adjMat ) { 
			sum += a;
		}
		if(sum>1) { 
			result = true;
		}
		return result;
	}

	public boolean checkForLeaves(int state) { 
		boolean hasLeaves = false;
		Node[] leaves;
		if(state==B){ 
			leaves = blueLeaf;
		} else {
			leaves = redLeaf;
		}
		for(Node n : leaves) { 
			if(n!=null) {
				hasLeaves = true;
			}
		}
		return hasLeaves;
	}

	public boolean blueSolution() {
		if( checkForLeaves(B) == false ) {
			return false;
		}
		for(Node n : blueRoot) { 
			if(n != null && n.state == B && explore(n,B) ) {
				return true;
			}
		}
		return false;
	}

	public boolean redSolution() {
		if( checkForLeaves(R) == false ) {
			return false;
		}
		for(Node n : redRoot) { 
			if(n != null && explore(n,R) ) {
				return true;
			}
		}
		return false;
	}

	public boolean explore(Node n, int state) { 
		if(n.leaf && n.state==state) { 
			// base case:
			// started at a root and reached a leaf
			return true;
		} else if( n == null ){
			// base case:
			// no root to begin with.
			return false;
		} else {
			// check neighbors
			for(Node e : n.neighbors) { 
				// if neighbor is null,
				// skip
				// otherwise,
				// if true, return true immediately
				if( e != null && explore(e,state) ) { 
					return true;
				}
			}
			return false;
		}
	}

	public void printResult(int num) {
		if( isValid() ) { 
			if( redSolution() ) {
				if( !hasParallelSolutions(R) ) {
					System.out.println("Case #"+num+": Red wins");
				} else {
					System.out.println("Case #"+num+": Impossible");
				}
			} else if( blueSolution()  ) { 
				if( !hasParallelSolutions(B) ) { 
					System.out.println("Case #"+num+": Blue wins");
				} else {
					System.out.println("Case #"+num+": Impossible");
				}
			} else {
				System.out.println("Case #"+num+": Nobody wins");
			}
		} else {
			System.out.println("Case #"+num+": Impossible");
		}
	}
}

class Node {
	HexBoard hexboard;
	Node[] neighbors;
	int row, col;
	int M;
	int state; // -1 = blue, 0 is empty, 1 is red
	boolean visited;
	boolean leaf;
	boolean root;

	public Node(int row, int col, HexBoard board) { 
		this.row = row;
		this.col = col;
		this.hexboard = board;
		this.neighbors = new Node[6];
	}

	public void link() {
		int n=0, s=1, e=2, w=3, ne=4, sw=5;

		if(row > 0) { 
			this.neighbors[n] = hexboard.getNode(row-1, col);
		} else {
			hexboard.redRoot[col] = this;
			this.root = true;
		}

		if(row < M) {
			this.neighbors[s] = hexboard.getNode(row+1, col);
		} else {
			hexboard.redLeaf[col] = this;
			this.leaf = true;
		}

		if(col > 0) { 
			this.neighbors[w] = hexboard.getNode(row, col-1);
		} else {
			hexboard.blueRoot[row] = this;
			this.root = true;
		}

		if(col < M) { 
			this.neighbors[e] = hexboard.getNode(row, col+1);
		} else {
			hexboard.blueLeaf[row] = this;
			this.leaf = true;
		}

		if(row > 1 && col < M) { 
			this.neighbors[ne] = hexboard.getNode(row-1, col+1);
		}

		if(row < M && col > 1) { 
			this.neighbors[sw] = hexboard.getNode(row+1, col-1);
		}
	}
}

