# Sudoku Checker

This code jam problem comes from the China New Grad 2014 Code Jam.

[Link](https://codejam.withgoogle.com/codejam/contest/2929486/dashboard#s=p0)

## Problem Description

This problem gives an input file with a series of sudoku puzzles.
Each sudoku puzzle consists of an M x M grid of sub-matrices,
each being an M x M sub-matrix itself. For example:

```plain
	5 3 4 6 7 8 9 1 2
	6 7 2 1 9 5 3 4 8
	1 9 8 3 4 2 5 6 7
	8 5 9 7 6 1 4 2 3
	4 2 6 8 5 3 7 9 1
	7 1 3 9 2 4 8 5 6
	9 6 1 5 3 7 2 8 4
	2 8 7 4 1 9 6 3 5
	3 4 5 2 8 6 1 7 9
```

Write a program that takes an input file containing a list 
of T sudoku solutions and checks if each solution is valid.
This requires checking if each of the numbers 1-M^2 appear 
once and only once per row, once and only once per column,
and once and only once per sub-matrix.

The input file consists of a first line that specifies T,
the total number of tests, followed by a set of T tests.
Each test has a first line that specifies M, the size of the puzzle,
followed by an M^2 x M^2 grid of integers.

## Solution Description

Solution procedure is as follows:
* We assume only that the file provides 
	correct number of tests, correct size of puzzle,
	and that each puzzle contains M^2 lines. 
* We iterate through the file one line at a time.
	We split/tokenize the line, check if it contains 
	the correct number of elements (M^2), and if so,
	add it to a 1D array of size M^4 (and if not,
	don't bother adding it to an array, just keep
	reading from the file until we get to the next puzzle).
* As we load each row of the puzzle, we also check the sum
	of each row to ensure each row is valid.
* Once the puzzle is loaded into a 1D array, 
	we check the sum of each column of the puzzle.
* Once we have checked the sum of each column, we check 
	the sum of each sub-matrix.

Notes on solution "tricks":
* One of the tricks is, not wasting too much time on error handling.
	There are many ways the input file could go wrong,
	but we keep it simple and make some assumptions.
	Can wrap this entire method with an `if(trust(filename)){` 
	if we are paranoid.
* We check each row as we load the puzzle into the 1D array
	so that we can save ourselves one additional pass through the puzzle.
* To check columns, sum up `puzzle[col + j*M*M]`, where `j=0..M*M`, for each column
* To check submatrices, start with the most efficient way to stride through
	the array (one element at a time, each element in order), then figure out 
	how to translate each index (there are M^4) into the submatrix row and column.
	We kept separate running sums for each submatrix row and column.

