import java.util.*;
import java.io.*;

/*
 * Google Code Jam: Sudoku Checker
 *
 * This problem comes from Google Code Jam,
 * China New Grad Test 2014.
 *
 * Implement a program to check the given input files
 * to see if they contain valid sudoku solutions.
 *
 * Link: https://codejam.withgoogle.com/codejam/contest/2929486/dashboard#s=p0
 */

public class SudokuChecker {
	public static void main(String[] args) throws FileNotFoundException { 
		String filename = "A-small-practice.in";
		if( verify(filename) ) { 
			long start = System.nanoTime();
			checkTest(filename);
			long finish = System.nanoTime();
			System.out.println("Seconds: " + (finish-start)/1E9);
		}
	}

	public static boolean verify(String filename) { 
		// 1001 lines for t = 100, n = 3, means 
		// file specs are met exactly
		return true;
	}

	public static void checkTest(String filename) throws FileNotFoundException { 
		Scanner s = new Scanner( new File(filename) );

		// The first line of the input gives the number of test cases, T. 
		int T = Integer.parseInt(s.nextLine());

		// T test cases follow. 

		// For each test case,
		for(int testNum = 1; testNum <= T; testNum++ ) {

			// Each test case starts with an integer M. 
			int M = Integer.parseInt( s.nextLine() );
			int M2 = M*M;
			int M4 = M2*M2;
			int Msum = (M2*(M2+1))/2;

			// Allocate a container for this test case
			int[] test = new int[M4];
			boolean broken = false;
			int testIndex = 0;
			
			// For each line in the test case
			for(int i = 0; i < M2; i++ ) { 

				// This is more awkward than using a scanner,
				// but is faster for checking if Ncols is consistent
				// (not always)
				// Tokenize the next line (assuming there are M2 lines)
				String row = s.nextLine();
				String[] tokens = row.split(" ");

				// use the broken boolean here. just trust me.

				// If not the right length, broken.
				if(tokens.length != M2) {
					broken = true;
				} 
				
				// If not broken, parse the row tokens 
				// and sum them up to check the row sum. 
				if(!broken) {
					int rowsum = 0;
					for(String token : tokens ) { 
						int tok = Integer.parseInt(token);
						rowsum += tok;
						test[testIndex] = tok;
						testIndex++;
					}
					
					System.out.println("Line: "+(i+1)+" Sum: "+rowsum);

					// If puzzle is correct, row sum should be Msum
					if(rowsum != Msum) {
						broken = true;
					}
				}
				
				// Loaded each line in the test,
				// checked it had the right number of tokens,
				// checked it had the right sum.
			}

			// If ok so far, check the puzzle (in earnest).
			if(!broken) { 
				checkPuzzle(test, M, M2, M4, Msum, testNum);
			} else {
				printFail(testNum);
			}
		}
	}// end check test

	public static void checkPuzzle(int[] puzzle, int M, int M2, int M4, int Msum, int testNum) { 

		boolean broken = false;

		// We already checked if sums of rows are valid. 

		// Check if sums of columns are valid.
		// Look at element COL, then COL + M^2, then COL + 2M^2, then COL + 3M^2, etc.
		for(int col = 0; col < M2; col++) { 
			// Check the sum of this column
			int colsum = 0;
			for(int j = 0; j < M2; j++ ) { 
				colsum += puzzle[col + j*M2];
			}
			if(colsum != Msum ) {
				broken = true;
			}

			System.out.println("Cols: "+(col+1)+" Sum: "+colsum);
		}

		// Check if sub-matrices are valid
		// Loop over each of the M^2 sub-matrices
		int[] subSums = new int[M2];
		for(int i = 0; i < M4; i++ ) { 

			int row = i/M2;
			int col = i%M2;

			int submatrixRow = row / M;
			int submatrixCol = col / M;

			subSums[submatrixRow*M + submatrixCol] += puzzle[i];
		}

		int i=1;
		for( int sum : subSums ) { 
			System.out.println("Submatrix: "+i+" Sum: "+sum);
			i++;
			if(sum != Msum) { 
				broken = true;
			}
		}

		if(broken) {
			printFail(testNum);
		} else {
			printSuccess(testNum);
		}

	} // end check puzzle

	public static void printFail(int testNum) { 
		System.out.println("Case #"+testNum+": No");
		System.out.println("-----------");
	}
	public static void printSuccess(int testNum) { 
		System.out.println("Case #"+testNum+": Yes");
		System.out.println("-----------");
	}
}
